import 'dart:async';

import 'package:agschat_example/constants.dart';
import 'package:agschat_example/view/chats_list_view.dart';
import 'package:agschat_example/view/widgets/comman.dart';
import 'package:get/get.dart';
import 'package:rxdart/subjects.dart';
import 'package:socket_io_client/socket_io_client.dart' as IO;
import 'package:socket_io_client/socket_io_client.dart';

class StreamSocket {
  StreamController<dynamic> socketResponse = BehaviorSubject();

  void Function(dynamic) get addResponse => socketResponse.sink.add;

  Stream<dynamic> get getResponse => socketResponse.stream;

  void dispose() {
    socketResponse.close();
  }
}

// 192.168.1.94:5000 local
// http://14.99.147.156:5000 // live
IO.Socket socket = IO.io('http://192.168.1.94:5000',
    OptionBuilder().setTransports(['websocket']).disableAutoConnect().build());
StreamSocket chatListSoket = StreamSocket();
StreamSocket messageSocket = StreamSocket();
StreamSocket profileViewSocket = StreamSocket();

void connectSocket() {
  socket.connect();
  socket.onConnect((_) {
    print('socket connect');
  });

  socket.on('group-list', (data) => chatListSoket.addResponse(data));
  socket.on('group-list', (data) => print(data));

  socket.onDisconnect((_) => print('disconnect'));
}

var params = {"secretKey": secKey, "_id": myUserId};

void getChatsList({forNewChat = false}) {
  socket.emit('get-groups', params);
  socket.on('group-list', (data) => chatListSoket.addResponse(data));
}

void joinGroup(grpId) {
  socket.emit('join', grpId);
}

void getPreviousMessages({grpId, id}) {
  var params = {"groupId": grpId, "_id": id};
  socket.emit('get-chat', params);
  socket.on('get-previous-chat', (data) => messageSocket.addResponse(data));
  socket.onDisconnect((_) => print('disconnect'));
}

void sendMessage(param) async {
  socket.emit('send-message', param);
}

void getUserProfile() {
  var params = {"userId": myUserId};
  socket.emit('get-profile', params);
  socket.on('profile-res', (data) => profileViewSocket.addResponse(data));
}

void updateProfile({name, profilePic}) {
  var params = {"userId": myUserId, "name": name, "profilePicture": profilePic};
  socket.emit("update-profile", params);

  socket.on("update-profile-res", (data) {
    if (data['isSuccess']) {
      Get.back();
    }
  });
}

void exitGroup({groupId, userId}) {
  var params = {
    "groupId": groupId,
    "userId": userId,
  };
  socket.emit("exit-group", params);
  socket.on("exit-group-res", (data) {
    if (data['isSuccess']) {
      Get.off(() => const ChatsListView());
    }
  });
}

void deleteGroup({groupId, userId}) {
  var params = {
    "groupId": groupId,
    "userId": userId,
  };
  socket.emit("delete-group", params);
  socket.on("delete-group-res", (data) {
    if (data['isSuccess']) {
      Get.off(() => const ChatsListView());
    }
  });
}

void clearChat({groupId, userId}) {
  var params = {
    "groupId": groupId,
    "userId": userId,
  };
  socket.emit("clear-chat", params);
  socket.on("clear-chat-res", (data) {
    if (data['isSuccess']) {
      getPreviousMessages(id: userId, grpId: groupId);
    }
  });
}

StreamSocket getUsersSocket = StreamSocket();
void getUsers() {
  var param = {'userId': myUserId};
  socket.emit("user-list", param);
  socket.on("user-list-res", (data) => getUsersSocket.addResponse(data));
}

void chat(param, {Function? success}) {
  print(param);
  socket.emit("create-group", {"groupDetails": param});

  socket.on("create-group-res", (data) {
    if (data['isSuccess']) {
      getChatsList(forNewChat: true);
      if (success == null) {
        Get.back();
        Common.getSnackToast(title: "Success", message: data['msg']);
      } else {
        Common.getSnackToast(title: "Success", message: data['msg']);
        success();
      }
    }
  });
}

void removeGroupMember(param, onSuccess()) {
  socket.emit("remove-member", param);
  socket.on("remove-member-res", (data) {
    print(data);
    if (data['isSuccess']) {
      onSuccess();
    }
  });
}

void getUnreadCount(groupId) {
  var param = {"userId": myUserId, "secretKey": secKey, "groupId": groupId};
  socket.emit("unread-count", param);
  socket.on("unread-count-res", (data) {
    print(data);
    getChatsList();
  });
}

void addMember(param) {
  socket.emit("add-member", param);
  socket.on("add-member-res", (data) {
    if (data['isSuccess']) {
      Get.back(result: param);
    }
  });
}

void updateGroupInfo(param) {
  socket.emit("update-group", param);
  socket.on("update-group-res", (data) {
    Get.offAll(() => const ChatsListView());
  });
}

void getGroupDetail(groupId, success(val)) {
  var param = {"secretKey": secKey, "groupId": groupId};
  socket.emit("get-group", param);
  socket.on("get-group-res", (data) {
    success(data);
  });
}

void getProfileDetail(success(val)) {
  var param = {"secretKey": secKey, "userId": myUserId};
  socket.emit("user-detail", param);
  socket.on("user-detail-res", (data) {
    success(data['data']);
  });
}
